/*

  GESTURES.DLL

  Loadmodule for Litestep shell (and hereby compatible clones)
  Copyright (C) 2001-2002 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/

  Programmed in LCC-WIN32 free C compiler/tools/IDE by Jacob Navia.
  Lcc-win32 compiler and accompanying tools may be obtained at:
  http://www.cs.virginia.edu/~lcc-win32/

  For another compilers some adjustments/corrections may be needed.


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <windows.h>
//#include <shellapi.h>
//#include <malloc.h>
#include <string.h>
#include "lsapi.h"
#include "main.h"
#include "./libstroke/stroke.h"
#include "hookdll.h"
#include "../procinfo/procinfo.h"

BOOL enabled;
BOOL reverse_sequence;
BOOL no_excludes;
BOOL no_def_excludes;

HWND hLitestep = NULL;
HWND hDesktop = NULL;
DWORD OSVer;
HINSTANCE DLLInstance = NULL;

void BangEnable(HWND Caller, char *Args)
{
	enabled = TRUE;
}

void BangDisable(HWND Caller, char *Args)
{
	enabled = FALSE;
}

void BangToggle(HWND Caller, char *Args)
{
	enabled = !enabled;
}

__declspec( dllexport ) int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	OSVERSIONINFO osvinfo;
	char buffer[MAX_LINE_LENGTH];

	// Get windows version
	osvinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if ( !GetVersionEx(&osvinfo) )
		return 1;
	OSVer = osvinfo.dwPlatformId;

	// Cache important window handles
	hLitestep = GetLitestepWnd();
	hDesktop = FindWindow("DesktopBackgroundClass", NULL);
	if (!hDesktop)
		hDesktop = GetDesktopWindow();

	DLLInstance = dllInst;

	// Set variables
	enabled = TRUE;
	send_motion = FALSE;
	stroke_draw = FALSE;
	reverse_sequence = FALSE;
	no_excludes = FALSE;
	no_def_excludes = FALSE;

	reverse_sequence = GetRCBool("StrokeReversed", TRUE);
	no_excludes =  GetRCBool("StrokeNoExcludes", TRUE);
	no_def_excludes =  GetRCBool("StrokeNoDefaultExcludes", TRUE);

	stroke_draw = GetRCBool("StrokeDrawing", TRUE);
	stroke_color = GetRCColor("StrokeColor", RGB(255, 0, 0));
	def_timeout = GetRCInt("StrokeTimeout", 500);
	LOGF("StrokeTimeout: %d", def_timeout);

	// Set mouse button that will activate stroke
	mbutton = MK_RBUTTON;
	switch ( GetRCInt("StrokeButton", 2) )
	{
		case 0:
			mbutton = -1;	// No mouse button. Hotkey should be set in this case.
			break;
		case 1:
			mbutton = MK_LBUTTON;
			break;
		case 2:
			mbutton = MK_RBUTTON;
			break;
		case 3:
			mbutton = MK_MBUTTON;
			break;
	}
	LOGF("mbutton: '%s'", mbutton == MK_LBUTTON ? "LEFT" :
						  mbutton == MK_RBUTTON ? "RIGHT" :
						  mbutton == MK_MBUTTON ? "MIDDLE" :
						  mbutton == -1 ? "NONE" :
						  "UNKNOWN VALUE");

	// Set hotkey.
	hotkey = 0;
	if ( GetRCString("StrokeHotkey", buffer, "", sizeof(buffer)) )
	{
		strupr(buffer);
		LOGF("hotkey: '%s'", buffer);
		if (strstr(buffer, "SHIFT"))
			hotkey |= HK_SHIFT;
		if (strstr(buffer, "ALT"))
			hotkey |= HK_ALT;
		if (strstr(buffer, "CTRL"))
			hotkey |= HK_CTRL;
//		LOGF("hotkey is '%x'", hotkey);
	}

/*	if ( GetRCString("StrokeAction", buffer, "", sizeof(buffer)) )
		setAction( &def_action, buffer );
	else
		setAction( &def_action, NULL );
*/
	// Load stroke definitions
	if ( !loadStrokes(szPath) )
	{
		return 1;
	};

	// Initialize process-information retriever
	if ( !initProcessInfo() )
	{
		return 1;
	}

	// Load system-wide hook library
	if ( !loadHookDLL( dllInst, NEED_HOOKDLL_VER ) )
	{
		return 1;
	}

	// Create tool-window
	if ( !createToolWindow() )
	{
		return 1;
	}

	// Allocate sound paths
	if ( GetRCString("StrokeNotifySound", buffer, "", sizeof(buffer)) )
		def_notify_snd = strdup(buffer);
	else
		def_notify_snd = NULL;

	if ( GetRCString("StrokeTimeoutSound", buffer, "", sizeof(buffer)) )
		def_timeout_snd = strdup(buffer);
	else
		def_timeout_snd = NULL;

	if ( GetRCString("StrokeExcludeSound", buffer, "", sizeof(buffer)) )
		def_exclude_snd = strdup(buffer);
	else
		def_exclude_snd = NULL;

	// Load list of *StrokeExcludeWin from step.rc. Build ex_win_list array.
	loadExcludedWindows();

	// Bang commands
	AddBangCommand("!GesturesEnable", &BangEnable);
	AddBangCommand("!GesturesDisable", &BangDisable);
	AddBangCommand("!GesturesToggle", &BangToggle);

	LOGF("MouseMsg: %d", MouseMsg);
	LOGF("Tool window HWND: #%x", ToolWindow);

	// Initialize Libstroke routines
	stroke_init();

	// Set mouse-hook for mouse movement. Set it for appropriate button too. If no StrokeButton is defined,
	// only mouse-movement is checked.
	setMouseHook( ToolWindow,
		EV_MOUSEWHEEL | EV_MOUSEMOVE | EV_NCMOUSEMOVE |
		(mbutton == MK_LBUTTON ? EV_LBUTTONDOWN | EV_LBUTTONUP | EV_NCLBUTTONDOWN | EV_NCLBUTTONUP : // Left button
		 mbutton == MK_RBUTTON ? EV_RBUTTONDOWN | EV_RBUTTONUP | EV_NCRBUTTONDOWN | EV_NCRBUTTONUP : // Right button (default)
		 mbutton == MK_MBUTTON ? EV_MBUTTONDOWN | EV_MBUTTONUP | EV_NCMBUTTONDOWN | EV_NCMBUTTONUP : // Middle button
		 0 // check no buttons
		)
	);

	// Set keyboard-hook only if standalone hotkey is used. If not, hotkey is tested if mouse-button
	// is pressed, so no keyboard hook is needed.
	if ( (hotkey != 0) && (mbutton == -1) )
	{
		LOG("Keyboard hook has been set!");
		setKeyboardHook( ToolWindow, EV_KEYDOWN | EV_KEYUP );
	}

	return 0;
}

__declspec( dllexport ) void quitModule(HINSTANCE dllInst)
{
	(unsetKeyboardHook)( ToolWindow );
	(unsetMouseHook)( ToolWindow );
	destroyToolWindow();

	RemoveBangCommand("!GesturesDisable");
	RemoveBangCommand("!GesturesEnable");
	RemoveBangCommand("!GesturesToggle");

	unloadHookDLL();

	if ( def_notify_snd )
		free(def_notify_snd);
	if ( def_timeout_snd )
		free(def_timeout_snd);
	if (ex_win_list)
		free(ex_win_list);

	freeStrokes( strokes );

	doneProcessInfo();
}

__declspec( dllexport ) int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	return initModuleEx (ParentWnd, dllInst, (wd)? wd->lsPath : NULL);
}


__declspec( dllexport ) BOOL WINAPI LibMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(lpvReserved);

	if (fdwReason == DLL_PROCESS_ATTACH)
	{
		LOG("DllMain called: DLL_PROCESS_ATTACH");
		DisableThreadLibraryCalls(hDLLInst);
	}
	return TRUE;
}
