/*

  This file is part of GESTURES.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#ifndef __MAIN_H
#define __MAIN_H

//#include "wharfdata.h"
#include "lsapi.h"

//#define LOGGING

#ifdef LOGGING
	char logmsg[256];
	LPVOID lpMsgBuf;

	#define LOG(TXT)		OutputDebugString(ST_NAME TEXT(": ") TXT "\n")
	#define LOGF(TXT,PRM)	sprintf(logmsg, ST_NAME TEXT(": ") TXT "\n", PRM); \
							OutputDebugString(logmsg)
	#define LOGIF(IF,TXT)	if (IF)	OutputDebugString(ST_NAME TEXT(": ") TXT "\n")
	#define LOGLE			FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), \
							MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL); \
							OutputDebugString(lpMsgBuf); \
							LocalFree( lpMsgBuf )
#else
	#define LOG(TXT)
	#define LOGF(TXT,PRM)
	#define LOGIF(IF,TXT)
	#define LOGLE
#endif

#define STRVALID(s) ((s != NULL) && (strlen(s) > 0))
#define NELEMENTS(a) (sizeof(a)/sizeof(a[0]))

#define	NEED_HOOKDLL_VER	0x0202

// strings
#define ST_SHORTNAME      TEXT("gestures")
#define ST_NAME           TEXT("GESTURES.DLL")
#define ST_VERSION        TEXT("1.1a")
#define ST_AUTHOR         TEXT("pavelvitis")
#define ST_REVID          ST_SHORTNAME TEXT(" ") ST_VERSION TEXT(" (") __DATE__ TEXT(" / ") __TIME__ TEXT(") ") ST_AUTHOR

// window classes
#define WC_LSDESKTOP      TEXT("DesktopBackgroundClass")
#define WC_MAIN           TEXT("LS_") ST_SHORTNAME

#define TOKEN		TEXT("*stroke")

#define ACT_NONE			0

#define ACT_WC_CLOSE		1
#define ACT_WC_MINIMIZE		2
#define ACT_WC_MAXIMIZE		3
#define ACT_WC_RESTORE		4
#define ACT_WC_SENDBACK		5
#define ACT_WC_PREV			6
#define ACT_WC_NEXT			7
#define ACT_WC_SHADE		8
#define ACT_WC_UNSHADE		9
#define ACT_WC_ONTOP		10
#define ACT_WC_NOTONTOP		11
#define ACT_WC_RESIZE		12
#define ACT_WC_MOVE			13

#define ACT_HELP_BACK		50
#define ACT_MESSAGE			51
#define ACT_BROWSER_BACK	52
#define ACT_BROWSER_FORWARD	53
#define ACT_BROWSER_RELOAD	54

#define ACT_EXECUTE			100
#define ACT_BANG			101

#define ALTKEY ((GetAsyncKeyState(VK_MENU) & 0x8000U) != 0)
#define CTRLKEY ((GetAsyncKeyState(VK_CONTROL) & 0x8000U) != 0)
#define LSHIFTKEY ((GetAsyncKeyState(VK_LSHIFT) & 0x8000U) != 0)
#define RSHIFTKEY ((GetAsyncKeyState(VK_RSHIFT) & 0x8000U) != 0)
#define SHIFTKEY ((GetAsyncKeyState(VK_SHIFT) & 0x8000U) != 0)

#define HK_SHIFT	0x0001U
#define HK_ALT		0x0002U
#define HK_CTRL		0x0004U

#define LMBUTTON ((GetAsyncKeyState(VK_LBUTTON) & 0x8000U) != 0)
#define RMBUTTON ((GetAsyncKeyState(VK_RBUTTON) & 0x8000U) != 0)
#define MMBUTTON ((GetAsyncKeyState(VK_MBUTTON) & 0x8000U) != 0)

typedef struct ACTION
{
	UINT id;
	RECT rect;
	LPSTR parm;
} ACTION;

typedef struct STROKE *PSTROKE;

struct STROKE
{
	char *name, *notify_snd, *timeout_snd, *exclude_win, *exclude_mod;
	char *sequence;
	ACTION action;
	UINT timeout;
	PSTROKE next;
} STROKE;

typedef struct EX_WIN
{
	LPSTR module;
	LPSTR class;
	LPSTR title;
} EX_WIN;

extern BOOL reverse_sequence;
extern BOOL stroke_draw;
extern BOOL no_excludes;
extern BOOL no_def_excludes;
extern COLORREF stroke_color;
extern BOOL enabled;
extern BOOL send_motion;
extern HWND ToolWindow;
extern HWND hLitestep;
extern HWND hDesktop;
extern char sequence[];
extern HINSTANCE DLLInstance;
extern char *def_notify_snd, *def_timeout_snd, *def_exclude_snd;
extern UINT def_timeout;
extern ACTION def_action;
extern PSTROKE strokes;
extern EX_WIN *ex_win_list;
extern UINT ex_win_cnt;
extern UINT mbutton;
extern UINT hotkey;

BOOL createToolWindow( void );
void destroyToolWindow( void );
void startTimer( UINT );
void stopTimer( void );

BOOL isAppWindow( HWND );
BOOL loadStrokes( LPCTSTR );
void loadExcludedWindows();

void freeStrokes( PSTROKE );
PSTROKE newStroke( void );
PSTROKE addStroke( PSTROKE );
void deleteStroke( PSTROKE );
void setAction( ACTION*, LPSTR, LPCTSTR );

BOOL excluded_window(LPSTR, LPSTR, LPSTR);
void startDesktopDrawing(int, int);
void desktopDrawPoint(int, int);
void stopDesktopDrawing(void);

void sound( LPCTSTR, LPCTSTR );

/*
#ifdef __cplusplus
extern "C" {
#endif

__declspec( dllexport ) int initModule( HWND, HINSTANCE, wharfDataType* );
__declspec( dllexport ) int initModuleEx( HWND, HINSTANCE, LPCSTR );
__declspec( dllexport ) void quitModule( HINSTANCE );
__declspec( dllexport ) BOOL WINAPI LibMain( HINSTANCE, DWORD, LPVOID );

#ifdef __cplusplus
}
#endif
*/

#endif	//__MAIN_H
