/*

  This file is part of GESTURES.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#include <windows.h>
#include "main.h"

BOOL stroke_draw = FALSE;
COLORREF stroke_color;
char *def_exclude_snd = NULL;

// Test if window is application window
BOOL isAppWindow( HWND wnd )
{
	DWORD nStyle;
	HWND hOwner, hParent;
	BOOL bResult;

	bResult = TRUE;

	nStyle = GetWindowLong( wnd, GWL_STYLE );
	if ( ((nStyle & WS_CHILD) != 0) || ((nStyle & WS_VISIBLE) == 0) )
		bResult = FALSE;

	nStyle = GetWindowLong( wnd, GWL_EXSTYLE );
	if ( (nStyle & WS_EX_TOOLWINDOW) != 0 )
		bResult = FALSE;

	hOwner = GetWindow( wnd, GW_OWNER );
	if ( (hOwner != NULL) && bResult )
		bResult = !isAppWindow( hOwner );

	hParent = GetParent( wnd );
	if ( hParent != NULL )
		bResult = FALSE;

	nStyle = GetWindowLong( wnd, GWL_USERDATA );
	if ( nStyle == magicDWord )
		bResult = FALSE;

	return bResult;
}

// Test if window parameters match any record in windows-exclusion-list,
// if so, return TRUE
BOOL excluded_window(LPSTR module, LPSTR class, LPSTR title)
{
	// No exclusion-list defined
	if ((no_excludes) || (ex_win_list == NULL) || (ex_win_cnt == 0))
		return FALSE;

	for (int i = 0; i < ex_win_cnt; i++)
	{
		BOOL matched = FALSE;
		// Test for module (executable) path
		if ( (ex_win_list[i].module != NULL) && (strlen(ex_win_list[i].module) > 0) )
		{
			if ((module != NULL) && match(ex_win_list[i].module, module))
				matched = TRUE;
			else
				continue;
		}

		// Test for class name
		if ( (ex_win_list[i].class != NULL) && (strlen(ex_win_list[i].class) > 0) )
		{
			if ((class != NULL) && match(ex_win_list[i].class, class))
				matched = TRUE;
			else
				continue;
		}

		// Test for window title
		if ( (ex_win_list[i].title != NULL) && (strlen(ex_win_list[i].title) > 0) )
		{
			if ((title != NULL) && match(ex_win_list[i].title, title))
				matched = TRUE;
			else
				continue;
		}

		// Match
		if (matched)
		{
			LOG("Excluded window match:");
			LOGF("  module: '%s'", module);
			LOGF("  class : '%s'", class);
			LOGF("  title : '%s'", title);

			// Play StrokeExcludeSound
			sound(NULL, def_exclude_snd);
			return TRUE;
		}
	}

	return FALSE;
}

/*
BOOL CALLBACK EnumProc(HWND hwnd,	LPARAM lParam)
{
	char classbuf[256];

	GetClassName(hwnd, classbuf, sizeof(classbuf));
	LOGF("Enumwnd: %s", classbuf);
	SendMessage(hwnd, WM_CANCELMODE, 0, 0);
	SendMessage(hwnd, WM_EXITSIZEMOVE, 0, 0);
//	SendMessage(hwnd, WM_CAPTURECHANGED, 0, 0);
	return TRUE;
}

void _WindowFromPoint( HWND wnd )
{
	char classbuf[256];
	HWND twnd, twnd1;
	DWORD thid;

	twnd = wnd;
	do {
		SendMessage(twnd, WM_CANCELMODE, 0, 0);
		SendMessage(twnd, WM_CAPTURECHANGED, 0, 0);
		twnd = GetWindow(twnd, GW_HWNDPREV);
	} while ( twnd && (twnd != wnd) );

	EnumChildWindows(wnd, EnumProc, 0);
*/


static HDC hDC = NULL;
static HPEN hPen = NULL;
static HPEN hOldPen = NULL;
static int nMode = 0;
static POINT LastPos;
static POINT drawbuf[1000];
static UINT drawpos = 0;

// Start of stroke drawing
void startDesktopDrawing(int x, int y)
{
	// No drawing enabled
	if (!stroke_draw)
		return;

	// Flush GDI
	GdiFlush();
    // Let the desktop window sort itself out
	Sleep(50);

	// if hwnd is null -> CON
	hDC = GetDC(NULL);

	// Pen size
	hPen = CreatePen(PS_SOLID, 1, stroke_color);

	// Pen mode - nondestructive XOR
	nMode = SetROP2(hDC, R2_NOTXORPEN /*COPYPEN*/);
	hOldPen = (HPEN) SelectObject(hDC, hPen);

	// Draw line
	MoveToEx(hDC, x, y, NULL);

	LastPos.x = x;
	LastPos.y = y;

	// Initialize draw path buffer
	drawpos = 0;
	drawbuf[drawpos].x = x;
	drawbuf[drawpos].y = y;
	drawpos++;
}

// Draw line from last cursor position to current
void desktopDrawPoint(int x, int y)
{
	// No drawing enabled
	if (!stroke_draw)
		return;

	// Drawing path buffer overrun, stop drawing
	if ( drawpos >= NELEMENTS(drawbuf) )
		return;

//	if ( (LastPos.x == x) && (LastPos.y == y) )
//		return;

	// Remember drawing path for later clean-up
	drawbuf[drawpos].x = x;
	drawbuf[drawpos].y = y;
	drawpos++;

	// Draw line
	LineTo(hDC, x, y);

	LastPos.x = x;
	LastPos.y = y;

	// Flush GDI, so the line is visible immediately
	GdiFlush();
}

/*
BOOL CALLBACK EnumProc(HWND wnd, LPARAM lParam)
{
	InvalidateRect(wnd, NULL, TRUE);
	SendMessage(wnd, WM_ERASEBKGND, 0, 0);
	SendMessage(wnd, WM_PAINT, 0, 0);
	return TRUE;
}
*/

// Stop stroke drawing
void stopDesktopDrawing(void)
{
	// No drawing enabled
	if (!stroke_draw)
		return;

//	nMode = SetROP2(hDC, R2_MASKNOTPEN /*COPYPEN*/);
//	LineTo(hDC, LastPos.x, LastPos.y);

	// Clear final segment
	MoveToEx(hDC, drawbuf[0].x, drawbuf[0].y, NULL);
	// Clear the line
	for ( int i = 0; i < drawpos; i++ )
		LineTo(hDC, drawbuf[i].x, drawbuf[i].y);
	// Make it visible right now
	GdiFlush();

	// Return previous pen mode
	SetROP2(hDC, nMode);
	SelectObject(hDC, hOldPen);

	DeleteObject(hPen);

	ReleaseDC(NULL,hDC);
//	MoveWindow(ToolWindow, 0, 0, 0, 0, FALSE);
//	EnumWindows(EnumProc, 0);
//	SendMessage(hDesktop, WM_PAINT, 0, 0);
}
