/*

Process info
Get information about running process.
Windows 9x / WindowsNT compatible.

*/
#include "../procinfo/procinfo.h"

// ToolHelp Function Pointers.
HANDLE (WINAPI *lpfCreateToolhelp32Snapshot)(DWORD,DWORD);
BOOL (WINAPI *lpfProcess32First)(HANDLE,LPPROCESSENTRY32);
BOOL (WINAPI *lpfProcess32Next)(HANDLE,LPPROCESSENTRY32);

// PSAPI Function Pointers.
BOOL (WINAPI *lpfEnumProcesses)( DWORD *, DWORD cb, DWORD * );
BOOL (WINAPI *lpfEnumProcessModules)( HANDLE, HMODULE *, DWORD, LPDWORD );
DWORD (WINAPI *lpfGetModuleFileNameEx)( HANDLE, HMODULE, LPTSTR, DWORD );

static HINSTANCE hInstLib = NULL;
static DWORD osv = 0;

BOOL initProcessInfo( void )
{
	OSVERSIONINFO osverinfo;
/*	HANDLE         hSnapShot ;
	PROCESSENTRY32 procentry ;
	BOOL           bFlag ;
	LPDWORD        lpdwPIDs ;
	DWORD          dwSize, dwSize2, dwIndex ;
	HMODULE        hMod ;
	HANDLE         hProcess ;
	char           szFileName[ MAX_PATH ] ;
	EnumInfoStruct sInfo ;
*/
	// Check to see if we're under Windows95 or Windows NT.
	osverinfo.dwOSVersionInfoSize = sizeof( osverinfo ) ;
	if( !GetVersionEx( &osverinfo ) )
	{
		// Something went wrong - can't get windows version
		return FALSE ;
	}
	osv = osverinfo.dwPlatformId;

	// If Windows NT:
	if( osv == VER_PLATFORM_WIN32_NT )
	{
		// Load library and get the procedures explicitly. We do
		// this so that we don't have to worry about modules using
		// this code failing to load under Windows 95, because
		// it can't resolve references to the PSAPI.DLL.
		hInstLib = LoadLibraryA( "PSAPI.DLL" );
		if( hInstLib == NULL )
			return FALSE;

		// Get pointers to needed functions
		lpfEnumProcesses = (BOOL(WINAPI *)(DWORD *,DWORD,DWORD*)) GetProcAddress( hInstLib, "EnumProcesses" );
		lpfEnumProcessModules = (BOOL(WINAPI *)(HANDLE, HMODULE *, DWORD, LPDWORD)) GetProcAddress( hInstLib, "EnumProcessModules" );
		lpfGetModuleFileNameEx =(DWORD (WINAPI *)(HANDLE, HMODULE, LPTSTR, DWORD )) GetProcAddress( hInstLib, "GetModuleFileNameExA" );
		if ( lpfEnumProcesses == NULL || lpfEnumProcessModules == NULL || lpfGetModuleFileNameEx == NULL )
		{
			FreeLibrary( hInstLib );
			return FALSE ;
		}

		//LOG("Windows NT/2K detected");
	}
	else if ( osv == VER_PLATFORM_WIN32_WINDOWS )
	{
		hInstLib = LoadLibraryA( "Kernel32.DLL" );
		if( hInstLib == NULL )
			return FALSE;
		// Get procedure addresses.
		// We are linking to these functions of Kernel32
		// explicitly, because otherwise a module using
		// this code would fail to load under Windows NT,
		// which does not have the Toolhelp32
		// functions in the Kernel 32.
		lpfCreateToolhelp32Snapshot = (HANDLE(WINAPI *)(DWORD,DWORD)) GetProcAddress( hInstLib, "CreateToolhelp32Snapshot" );
		lpfProcess32First = (BOOL(WINAPI *)(HANDLE,LPPROCESSENTRY32)) GetProcAddress( hInstLib, "Process32First" );
		lpfProcess32Next = (BOOL(WINAPI *)(HANDLE,LPPROCESSENTRY32)) GetProcAddress( hInstLib, "Process32Next" );
		if( lpfProcess32Next == NULL || lpfProcess32First == NULL || lpfCreateToolhelp32Snapshot == NULL )
		{
			FreeLibrary( hInstLib ) ;
			return FALSE ;
		}
		//LOG("Windows 9x detected");
	}

	// Now we have needed functions prepared for use
	return TRUE;
}

void doneProcessInfo( void )
{
	if ( hInstLib != NULL )
		FreeLibrary( hInstLib );
}

// Get filename of process that created window
LPCTSTR getWndFilename( HWND wnd, char *fname, WORD maxlen  )
{
	DWORD hid;

	// Not a window
	if (!wnd)
		return NULL;

	if (GetWindowThreadProcessId(wnd, &hid) == 0)
		return NULL;

	if ( osv == VER_PLATFORM_WIN32_NT )
	{
		HANDLE hHandle;
		HMODULE hModule;
		DWORD dwSize;
		char tfname[MAX_PATH];

		hHandle = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, hid);
//		LOGF("ProcessHandle: %d", hHandle);

		lpfEnumProcessModules(hHandle, &hModule, sizeof(hModule), &dwSize);
//		LOGF("ProcessModule: %d", hModule);
		if ( lpfGetModuleFileNameEx(hHandle, hModule, tfname, MAX_PATH) > 0 )
		{
//			LOGF("Process filename: '%s'", tfname);
			CloseHandle(hHandle);
			return (strncpy(fname, tfname, maxlen));
		}

		CloseHandle(hHandle);
	}
	else if ( osv == VER_PLATFORM_WIN32_WINDOWS )
	{
		HANDLE hHandle;
		PROCESSENTRY32 data;

		hHandle = lpfCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
		data.dwSize = sizeof(data);
		if (lpfProcess32First(hHandle, &data))
		do {
			if (data.th32ProcessID == hid)
			{
				CloseHandle(hHandle);
				return (strncpy(fname, data.szExeFile, maxlen));
			}
		} while (lpfProcess32Next(hHandle, &data));

		CloseHandle(hHandle);
	}

	return NULL;
}
